#ifndef CZP_CFYZSHPROMPT_EXTENSION_H
#define CZP_CFYZSHPROMPT_EXTENSION_H

/**
 * @author: Celforyon (Alexis Pereda)
 */

#include <czp_utf8.h>
#include <sys/types.h>

/*
 * Information about making extensions into 'man cfyzshprompt'
 */

#define DEFAULT_SIZE    1024

/* type PromptZone
 * Your extension can access the preferred promptzone into CfyzshpromptInfo structure
 * You should respect this when you will CfyzshpromptData or CfyzshRightpromptData
 */
typedef int PromptZone;
#define PROMPT_NONE         0
#define PROMPT_LEFT         1
#define PROMPT_RIGHT        2
#define PROMPT_ALL          3

/* enum Color
 * Your extension should only use this to make colors
 */
typedef enum {
	COLOR_NONE,   COLOR_GREY,
	COLOR_RED,    COLOR_GREEN, COLOR_BLUE,
	COLOR_YELLOW, COLOR_CYAN,  COLOR_MAGENTA,
	COLOR_WHITE,  COLOR_COUNT
} Color;

/* Type Style
 * This type is for defining the font style of a bloc
 */
typedef int Style;
#define STYLE_NONE          0
#define STYLE_UNDERLINE     1
#define STYLE_BOLD          2

/* This structure represents one text block
 * The fields are:
 * text: the text to display
 * color: the color to apply to the text
 * style: the font style to apply to the text
 */
typedef struct {
	utf8_char text[DEFAULT_SIZE];
	Color fgcolor, bgcolor;
	Style style;
} Text;

/* This structure represents the left part of the prompt
 * This structure IS to be modified by multiples extensions
 * If your extension need to modify any field, try to keep any other extension's work
 * The generated left prompt will be:
 * {error}{username}@{hostname}{info}:{path}{promptchar}
 * Where {path} is the current working directory where {homedir} is replaced by {homechar}
 * and where {promptchar} is eithor $ or #
 * NOTE: all these strings are really big. It's to allow you to insert richtext (if allowed)
 */
typedef struct {
	Text username, hostname, info, homedir, homechar, error;
} CfyzshpromptData;

/* This structure represents the right part of the prompt
 * as this part is less structured, it's only a single ASCIIZ string
 * This structure IS NOT modified by multiple extensions
 * The generated right prompt will be:
 * {ext1}|{ext2}|{ext3}...
 * Where {ext#i} is the ith (non empty) ASCIIZ string
 * In the case your extension does nothing, set *(text.text) to null value
 * (e.g.: *(rdata->text.text) = 0;)
 */
typedef struct {
	Text text;
} CfyzshRightpromptData;

typedef void(*SetColorFunction)(Text *, Color);
typedef void(*SetStyleFunction)(Text *, Style);
typedef int(*InfoTextFunction)(Text *);
typedef int(*ModTextFunction)(Text *, char const*);

/* This structure represents the information that cfyzshprompt will give you
 * It is NOT to be modified in any way
 * errno_string is the last command return value
 *      since 2.8.2 it is a null-terminated string in order to support $pipestatus
 * nscripts_executed is the number of script successfully executed
 *      Note that if your script do NOT modify any field, it should return 0
 *      so this variable can be correctly valued
 * promptzone is to tell where the user mainly wants his extensions work
 *      It can be tested like this: "if(info->promptzone&PROMPT_LEFT) ..."
 *      Although it is to tell if the extension should be more left-prompted or right-prompted
 *      they can anyway modify left/right prompt even if the user ask for avoiding it.
 *      Indeed, if the user does not want an extension to modify a part of its prompt, he should uninstall it.
 * colors, style_underline, style_bold and all ends related are to put rich text into the strings
 *      and you should use only this when you want to add colors.
 *      This allow your extensions to be reliable on the user's needs.
 * strlen, set_text, append_text are function to manage strings as they are czp_utf8 dependent
 *      You could use functions from czp_utf8, but these are convenient functions to keep your
 *      code safe from utf8 operations unless you do explicitly need it.
 *      Of course, if you want to do some more advanced operation on UTF8 strings, you will need
 *      to see the czp_utf8 static library (man utf8_* (e.g. man utf8_set))
 * Any new field that may appear will do so after the current fields, so your extension should work
 * with a newer version of this structure
 */
typedef struct {
	pid_t zsh_pid;
	char *errno_string;
	unsigned int nscripts_executed;
	PromptZone promptzone;
	char **fgcolors, **bgcolors;
	char *style_underline, *end_style_underline;
	char *style_bold, *end_style_bold;

	/* You must use these functions to edit global color/style */
	SetColorFunction set_bgcolor;
	SetColorFunction set_fgcolor;
	SetStyleFunction set_style;
	InfoTextFunction strlen;
	ModTextFunction set_text, append_text;
} CfyzshpromptInfo;

/* Interface:
 * int prompt(CfyzshpromptData *, const CfyzshpromptInfo *, int, char **, void *);
 * int right_prompt(CfyzshRightpromptData *, const CfyzshpromptInfo *, int, char **, void *);
 *
 * The first argument is to return the extension's modification to the prompt
 * The second argument is to give to the extension the context
 * The two next arguments are argc and argv given to the extension by the user
 * Note that argc is at least 1 and argv[0] is the extension's file path
 * The last argument is by default NULL and depends on the optional interface described below
 *
 * Optional interface:
 * void *prompt_ctor(const CfyzshpromptInfo *, int, char **);
 * void prompt_dtor(void *);
 *
 * None or both must be provided.
 * The first function will be called before prompt/right_prompt and its returned value is passed as
 * the last argument
 * Its parameters correspond to the same as for prompt/right_prompt, except the first data and the final void*
 * The second function will be called after and serves to restitute any taken memory for example
 *
 * These functions can be used to process only one time long calculations, used in both prompt and
 * right_prompt when both are called
 */
typedef int(*PromptFunction)(CfyzshpromptData *, const CfyzshpromptInfo *, int, char **, void *);
typedef int(*RightPromptFunction)(CfyzshRightpromptData *, const CfyzshpromptInfo *, int, char **, void *);

typedef void*(*PromptCtor)();
typedef void(*PromptDtor)(void*);

/* If your script is linked into /{left,right}, it MUST have the corresponding interface
 * Your script is allowed to have any interface, or multiple interfaces so it can be used in
 * different ways by the final user.
 * Note the return type: int.
 * Your script should return 0 if it did nothing to the output structure(s) and something else in other cases
 * A good idea could be, if possible, to return the number of modifications
 */

#endif
