#ifndef CZP_UTF8_H
#define CZP_UTF8_H

/**
 * @brief UTF8 library for CZP
 * @author: Celforyon (Alexis Pereda)
 */

/* Currently, UTF-8 is defined to use at most 4 bytes */
#define CZP_UTF8_MAX_BYTES    4

/* An UTF-8 char / string
 * To reduce the time cost of UTF-8, there is no dynamic allocation
 * so each character takes maximum memory size.
 */
typedef struct { char u[CZP_UTF8_MAX_BYTES];  } utf8_char;
typedef utf8_char*                              utf8_string;

/**
 * @brief displays an UTF-8 char
 *
 * @param[in] c an UTF-8 char
 */
void utf8_putc(utf8_char c);

/**
 * @brief displays an UTF-8 string
 *
 * @param[in] str an UTF-8 string
 */
void utf8_puts(utf8_string str);

/**
 * @brief displays an UTF-8 char for ZSH (surround by %1{ } if not ASCII)
 *
 * @param[in] c an UTF-8 char
 */
void utf8_zputc(utf8_char c);

/**
 * @brief displays an UTF-8 string for ZSH (call utf8_zputc)
 *
 * @param[in] str an UTF-8 string
 */
void utf8_zputs(utf8_string str);

/**
 * @brief sets an UTF-8 char from an UTF-8-ASCIIZ string.
 *
 * This string must starts with a valid UTF-8 character
 * This function will return the number of char read to make the UTF-8 char
 * and 0 if it was a NULL character
 *
 * @param[out] c a pointer to an UTF-8 char
 * @param[in] s the UTF-8-ASCIIZ string
 *
 * @return the number of char read from s
 */
int utf8_setc(utf8_char *c, char const*s);

/**
 * @brief sets an UTF-8 string from an UTF-8-ASCIIZ string.
 *
 * This string must starts with a valid UTF-8 character
 * and contains only valid UTF-8 characters in sequence
 * This function will return the number of char read from s
 *
 * @param[out] us an UTF-8 string
 * @param[in] s an UTF-8-ASCIIZ string
 *
 * @return the number of char read from s
 */
int utf8_set(utf8_string us, char const*s);

/**
 * @brief acts like utf8_set but stopping at nth UTF-8 char
 *
 * @param[out] us an UTF-8 string
 * @param[in] s an UTF-8-ASCIIZ string
 * @param[in] n the maximum number of UTF-8 char to read
 *
 * @return the number of char read from s
 */
int utf8_nset(utf8_string us, char const*s, int n);

/**
 * @brief concats an UTF-8-ASCIIZ string to an UTF-8 string
 *
 * See utf8_set for the requirements
 *
 * @param[out] us an UTF-8 string
 * @param[in] s an UTF-8-ASCIIZ string
 *
 * @return the number of char read from s
 */
int utf8_cat(utf8_string us, char const*s);

/**
 * @brief acts like utf8_cat but stopping at nth char
 *
 * @param[out] us an UTF-8 string
 * @param[in] s an UTF-8-ASCIIZ string
 * @param[in] n the maximum number of UTF-8 char to read
 *
 * @return the number of char read from s
 */
int utf8_ncat(utf8_string us, char const*s, int n);

/**
 * @brief copies from in to
 *
 * @param[out] to destination UTF-8 string
 * @param[in] from source UTF-8 string
 */
void utf8_setcu(utf8_char *to, utf8_char const*from);

/**
 * @brief copies from in to
 *
 * warning: to must be at least sized to from
 *
 * @param[out] to destination UTF-8 string
 * @param[in] from source UTF-8 string
 */
void utf8_setu(utf8_string to, utf8_string from);

/**
 * @brief copies from in to
 *
 * acts like utf8_set8 but stopping at nth char
 *
 * @param[out] to destination UTF-8 string
 * @param[in] from source UTF-8 string
 * @param[in] n the maximum number of UTF-8 char to read
 */
void utf8_nsetu(utf8_string to, utf8_string from, int n);

/**
 * @brief copies from at the end of to
 *
 * warning: to must have at least the size of from after last not null char
 *
 * @param[out] to destination UTF-8 string
 * @param[in] from source UTF-8 string
 */
void utf8_catu(utf8_string to, utf8_string from);

/**
 * @brief acts like utf8_cat8 but stopping at nth char
 *
 * @param[out] to destination UTF-8 string
 * @param[in] from source UTF-8 string
 * @param[in] n the maximum number of UTF-8 char to read
 */
void utf8_ncatu(utf8_string to, utf8_string from, int n);

/**
 * @brief splits us by sep and return substring after each call
 *
 * @param[in,out] us an UTF-8 string
 * @param[in] sep an UTF-8 char
 * @param[out] s the number of UTF-8 char read
 *
 * @return an UTF-8 substring
 */
utf8_string utf8_sep(utf8_string us, utf8_char sep, int *s);

/**
 * @brief returns the number of utf8_char before the null one
 *
 * @param[in] us an UTF-8 string
 *
 * @return the length of the string
 */
int utf8_strlen(utf8_string us);

/**
 * @brief returns 0 if a and b are equal
 *
 * @param[in] a an UTF-8 char
 * @param[in] b an UTF-8 char
 *
 * @return 0 if a == b
 */
int utf8_ccmp(utf8_char a, utf8_char b);

/**
 * @brief returns 0 if a and b are equal
 *
 * @param[in] a an UTF-8 string
 * @param[in] b an UTF-8 string
 *
 * @return 0 if a == b
 */
int utf8_cmp(utf8_string a, utf8_string b);

/**
 * @brief returns 0 if n first chars are equal
 *
 * @param[in] a an UTF-8 string
 * @param[in] b an UTF-8 string
 * @param[in] n maximum utf8_char to compare
 *
 * @return 0 if a == b until nth utf8_char
 */
int utf8_ncmp(utf8_string a, utf8_string b, int n);

/**
 * @brief returns 1 if us starts with s
 *
 * @param[in] us an UTF-8 string
 * @param[in] s an UTF-8 string
 *
 * @return 1 if us startis with s
 */
int utf8_startswith(utf8_string us, utf8_string s);

#endif
